/**
* The contents of this file are subject to the Mozilla Public 
* License Version 1.1 (the "License"); you may not use this 
* file except in compliance with the License. You may obtain a
* copy of the License at http://www.mozilla.org/MPL/
*
* Software distributed under the License is distributed on an 
* "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express 
* or implied. See the License for the specific language 
* governing rights and limitations under the License.
*
* The Original Code is this file as it was released on
* Sept 18, 2005.
*
* The Initial Developer of the Original Code is Terry Yuen. 
* Portions created by Terry Yuen are Copyright (C) 2005.
* All Rights Reserved.
**/

const YMailNotifier = {
  startup: function() {
    Components.classes["@mozilla.org/observer-service;1"]
        .getService(Components.interfaces.nsIObserverService)
        .addObserver(YMailNotifier, "ymailnotifier-status", false);

    if(!YMailNotifier.pref().getBoolPref("ymailnotifier.hideicon")) {
      document.getElementById("ymailnotifier-statusbar-button").setAttribute("showicon", true);
    }

    YMailNotifier.doAction("mail-count");
  },

  observe: function(subject, topic, data) {
    if(topic != "ymailnotifier-status")return;

    var indicator = document.getElementById("ymailnotifier_broadcast");

    data = data.split("@");
    switch(data[0]) {
      case "0":
        if(data[1] == "Mail Count") {
          YMailNotifier.setLoggedIn(true);
          indicator.setAttribute("value", data[2]);
          if(indicator.getAttribute("state") == "nologin") {
            indicator.removeAttribute("state");
          }
        }
        break;

      case "1":
        indicator.setAttribute("state", "checking");
        indicator.setAttribute("tooltiptext", data[1]);
        break;

      case "2":
        indicator.removeAttribute("state");
        indicator.setAttribute("value", data[1]);
        indicator.setAttribute("tooltiptext", data[2]);
        break;

      case "3":
        YMailNotifier.setLoggedIn(false);
        indicator.setAttribute("state", "nologin");
        indicator.removeAttribute("value");
        indicator.setAttribute("tooltiptext", data[1]);
        break;

      case "4":
      case "5":
        indicator.setAttribute("state", "error");
        indicator.setAttribute("tooltiptext", data[1]);
        break;
    }
  },

  setLoggedIn: function(loggedIn) {
    document.getElementById("ymailnotifier-checknow-item").setAttribute("disabled", !loggedIn);
    document.getElementById("ymailnotifier-login-item").setAttribute("hidden", loggedIn);
    document.getElementById("ymailnotifier-inbox-item").setAttribute("hidden", !loggedIn);
  },

  fillFrequencyMenus: function(event) {
    var f = YMailNotifier.pref().getIntPref("ymailnotifier.timer");

    f = f/60000;
    var arr = (f == 0) ? [0] : [5, 10, 20, 30, 60, 120];
    var f2 = null;

    for(var i=0; i<arr.length; i++) {
      if(f <= arr[i]) {
        f2 = arr[i];
        break;

      } else if(i + 1 == arr.length) {
        f2 = arr[i];
        break;

      } else if(f < arr[i + 1]) {
        if(f - arr[i] < arr[i + 1] - f) {
          f2 = arr[i];
        } else {
          f2 = arr[i + 1];
        }
        break;
      }
    }

    f2 = new String(f2);
    var elm = event.target.firstChild;
    while(elm != null) {
      if(elm.getAttribute("freq") == f2) {
        elm.setAttribute("checked", true);
        break;
      }
      elm = elm.nextSibling;
    }
  },

  fillAlerterMenus: function(event) {
    var prefs = YMailNotifier.pref();

    var a = prefs.getIntPref("ymailnotifier.alert");
    document.getElementById("ymailnotifier-sound-item").setAttribute("checked", ((a & 0x1) > 0));
    document.getElementById("ymailnotifier-alert-item").setAttribute("checked", ((a & 0x2) > 0));
    document.getElementById("ymailnotifier-hideicon-item").setAttribute("checked", prefs.getBoolPref("ymailnotifier.hideicon"));

    if(prefs.prefHasUserValue("ymailnotifier.external_app")) {
      var file = prefs.getComplexValue("ymailnotifier.external_app", Components.interfaces.nsILocalFile);

      if(file) {
        var elm = document.getElementById("ymailnotifier-app-item");
        elm.setAttribute("label", elm.getAttribute("labelfullbefore") + file.leafName + elm.getAttribute("labelfullafter"));
        elm.setAttribute("checked", true);
      }
    } else {
      var elm = document.getElementById("ymailnotifier-app-item");
      elm.setAttribute("label", elm.getAttribute("labelempty"));
      elm.setAttribute("checked", false);
    }
  },

  checkMiddleClick: function(event) {
    if(event.button == 1) {
      switch(event.target.id) {
        case "ymailnotifier-login-item":
        case "ymailnotifier-inbox-item":
        case "ymailnotifier-statusbar-button":
        case "ymailnotifier-toolbar-button":
          this.gotoInbox(event);
      }
    }
  },

  clickItem: function(event) {
    if(!event.target.id && event.target.hasAttribute("freq")) {
      var f = parseInt(event.target.getAttribute("freq"));

      YMailNotifier.pref().setIntPref("ymailnotifier.timer", f*60000);

      YMailNotifier.doAction("update-timer");
      return;
    }

    switch(event.target.id) {
      case "ymailnotifier-toolbar-button":
        if(event.target.getAttribute("state") == "nologin" ||
           parseInt(event.target.getAttribute("value")) > 0 ||
           event.shiftKey || event.ctrlKey || event.metaKey) {
          YMailNotifier.doAction("mail-count");
          this.gotoInbox(event);
          break;
        }
      case "ymailnotifier-checknow-item":
        YMailNotifier.doAction("check-now");
        break;

      case "ymailnotifier-sound-item":
      case "ymailnotifier-alert-item":
        var isSound = (event.target.id == "ymailnotifier-sound-item");
        var other = document.getElementById((isSound) ? "ymailnotifier-alert-item" : "ymailnotifier-sound-item");

        var a = (event.target.getAttribute("checked") == "true") ? 0 : (isSound) ? 0x1 : 0x2;
        a |= (other.getAttribute("checked") != "true") ? 0 : (isSound) ? 0x2 : 0x1;

        YMailNotifier.pref().setIntPref("ymailnotifier.alert", a);

        YMailNotifier.doAction("update-alerter");
        break;

      case "ymailnotifier-app-item":
        if(event.target.getAttribute("checked") == "true") {
          YMailNotifier.pref().clearUserPref("ymailnotifier.external_app");

        } else {
          var file = YMailNotifier.showFileSelector();
          if(file) {
            YMailNotifier.pref().setComplexValue(
                "ymailnotifier.external_app", 
                Components.interfaces.nsILocalFile,
                file
            );
          }
        }
        break;

      case "ymailnotifier-hideicon-item":
        var hide = (event.target.getAttribute("checked") != "true");
        YMailNotifier.pref().setBoolPref("ymailnotifier.hideicon", hide);

        var tmp = document.getElementById("ymailnotifier-statusbar-button");
        if(hide)tmp.removeAttribute("showicon"); else tmp.setAttribute("showicon", true);
        break;

      case "ymailnotifier-login-item":
      case "ymailnotifier-inbox-item":
        this.gotoInbox(event);
        break;
    }
  },

  onpopupshowing: function(event) {
    if(event.target.id != "ymailnotifier-main-menupopup")return;

    YMailNotifier.doAction("mail-count");

    var menu = document.getElementById("ymailnotifier-inbox-item");
    var n = document.getElementById("ymailnotifier_broadcast").getAttribute("value");
    if(n) {
      menu.setAttribute("label", menu.getAttribute("labelbefore") + n + menu.getAttribute("labelafter"));
    } else {
      menu.setAttribute("label", menu.getAttribute("labelempty"));
    }
  },

  gotoInbox: function(event) {
    var url = YMailNotifier.pref().getCharPref("ymailnotifier.inbox_url");

    openUILink(url, event);  //requires "utilityOverlay.js"
  },

  doAction: function(s) {
    Components.classes["@mozilla.org/observer-service;1"]
        .getService(Components.interfaces.nsIObserverService)
        .notifyObservers(null, "ymailnotifier-action", s);
  },

  showFileSelector: function() {
    const FP = Components.interfaces.nsIFilePicker;
    var p = Components.classes["@mozilla.org/filepicker;1"].createInstance(FP);
    p.init(window, "", FP.modeOpen);
    p.appendFilters(FP.filterApps);
    return (p.show() == FP.returnOK) ? p.file : null;
  },

  pref: function() {
    return Components.classes["@mozilla.org/preferences-service;1"]
        .getService(Components.interfaces.nsIPrefBranch);
  },

  onDragOver: function(event, flavor, session) {
    session.canDrop = true;
    session.dragAction = Components.interfaces.nsIDragService.DRAGDROP_ACTION_LINK;
    event.preventBubble();
  },

  onDrop: function(event, data, session) {
    var email = data.data.match(/[\w+-\.]+@[\w+-.]+\w/);
    if(email) {
      openUILink("http://compose.mail.yahoo.com/?To=" + encodeURIComponent(email), event);
    }
    event.preventBubble();
  },

  getSupportedFlavours: function() {
    var f = new FlavourSet();
    f.appendFlavour("text/unicode");
    f.appendFlavour("text/x-moz-url");
    return f;
  },

  destroy: function() {
    Components.classes["@mozilla.org/observer-service;1"]
        .getService(Components.interfaces.nsIObserverService)
        .removeObserver(YMailNotifier, "ymailnotifier-status");
  },

  QueryInterface: function(iid) {
    if(!iid.equals(Components.interfaces.nsIObserver) &&
        !iid.equals(Components.interfaces.nsISupports))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return YMailNotifier;
  }
};

window.addEventListener("load", YMailNotifier.startup, false);
window.addEventListener("unload", YMailNotifier.destroy, false);